//////////////////////////////////////////////////////////////////////
//
// ESP definitions
//
//////////////////////////////////////////////////////////////////////


/*! \file ESP.h
    \brief Header file for ESP.C
*/

#ifndef _ESP_H_
#define _ESP_H_

#include <fstream>
#include <vector>
#include <deque>
#include "NE.h"
#include "Population.h"


#define MAX_NUM_AGENTES_PERMIT 31

class Neuron;
class Environment;
class Network;

//! User defined parameters for ESP
struct espArgs 
{
  int numpops;
  int popsize;
  int numNets;  // Anadido: indica el numero de redes neuronales: una por cada actuador y sensor
  Network *net[MAX_NUM_AGENTES_PERMIT]; // Modificado: originalmente era Network *net. Ahora introduzco un array de punteros a redes (una para cada sensor/actuador)
  int seed; 
  int stagnation;
  bool grownets;
  bool minimize;
  double mutation;
  bool verbose;
  bool seednet;
  char netfile[100];
  std::string popfile;
  bool savenets;
  bool savepops;
};


typedef Population<Neuron> SubPop;


//! Esp class
class Esp : public NeuroEvolution {

private:
  std::vector< SubPop* > subPops[MAX_NUM_AGENTES_PERMIT];  //!< std::vector of points to the subpopulations.

public:
  // constructors
  Esp(struct espArgs *, Environment &);   // Modificado
  ~Esp();

  // La funcion original (aunque tambien la he modificado)
  Network* genNetwork() { return network[0]->newNetwork(inputDimension, 
						    numPops, 
						    outputDimension); }
	// La creada por mi
	Network* genNetwork(int n) { return network[n]->newNetwork(inputDimension, 
						    numPops, 
						    outputDimension); }						
  int getMinUnits() { Network *n = genNetwork(); 
                     int tmp = n->getMinUnits(); delete n; return tmp;}
  void echoParams();
  void create(); // creates a random population of Neurons
  void evolve(int);
  void recombine();
  void addSubPop();
  void endEvolution();
  void setMutation(double);
  void activateBurstMutation();
  double getAveFit() { return averageFitness; }
  bool incorporateNeurons(Network *);

  // Accessors
  double getBestFit(){  return phaseBest[0]->getFitness(); } //modificado
  int getNumPops() { return numPops; }
  int getGeneSize() { return geneSize; }
  void savePopulation(char *);
  Neuron* getNeuron(int i, int j, int n) { return subPops[n][i]->getIndividual(j); }
  int getSubPopSize() { return subPopSize; }
  int getGeneration() { return generation; }

private:
  Network *network[MAX_NUM_AGENTES_PERMIT];	// Modificado		Original: Network &network;
  double mutationRate;  //!< Rate of mutation. 
  int numPops;          //!< Number of subpopulations
  int numNets;			// Numero de redes neuronales: una por sensor/actuador
  int subPopSize;       //!< Number of neurons in each subpopulation.
  int numTrials;        //!< Number of evaluations each generation.
  int generation;       //!< Counter for the number of generations.
  double prevBest;
   
  //  bool forceBurstMutation;
  std::deque<double> perfQ;   //!< A queue of the best fitness from each gen.
  double averageFitness; //!< The average fitness for a generation.
  int geneSize;          //!< The length of the neuron chromosomes. \todo move to NE?
  unsigned int stagnation;        //!< Then number of generations w/out improvement before action is taken.
  bool recombineP;       //!< Whether to recombine. \todo change name
  bool verbose;
  
  bool growNets;

  bool saveNets;
  bool savepopspergeneration; // added by RT to allow saving pops after each generation
  std::ofstream graphfile;

  std::vector< Network* > bestNetwork; //!< \todo make private after debug
  std::vector< Network* > phaseBest;  // Modificado


  Esp(const Esp &); // just for safety
  Esp &operator=(const Esp &);

  void evalPop();
  void setGeneSize();
  void setBestNetwork(Network *,int);
  void handleStagnation();
  void burstMutate(int);  
  void burstMutate(Network *,int);
 // comentada un momento int removeSubPop(int);
  int removeSubPop( std::vector< Network* > pNet);
  int removeSubPop(int );
  void printStats();
  void doPlotting();
  int lesionTest(std::vector< Network* > pNet);
  void loadPopulation(const char *);
};



#endif
