/*! \file MultiAgent.h
 */

#ifndef _MULTIAGENT_H_
#define _MULTIAGENT_H_ 

#include <vector>
#include <deque>
#include "Environment.h"
#include "NetworkConnector.h" // funciones de comunicacion RF con el robot Aibo

struct multiArgs {
  char modelfile[100]; 
  double gain;
};

class Network;
//! Multi Agent robot control
/*! 
    Implements the control of an autonomous robot formed by
	n sub-agents, each one controlled by its own neural net
*/

class MultiAgent : public Environment {
public:
	MultiAgent (struct multiArgs *);
	bool useModel;
	virtual ~MultiAgent(){}
	void nextTask() {}   
	virtual void simplifyTask() {}
	virtual double evalNetDump(Network *net, FILE *) { return 0.0;} 
	virtual double generalizationTest(Network *) { return 0.0;}
	double state[6]; //<! \todo  make private again!


private:
	int numAgents;
	bool initialized;
	bool reset;
	std::deque< double* > stateQ;
	double noise;
	double trajNoise;
	double gain; // NEW_EXP
  
	//virtual double evalNet(Network *Net1, Network *Net2);
	virtual double evalNet(std::vector< Network* > pNet);
	
	void InitializeJoints (float *jointNewPosition);
	void CheckJointLimits (float *jointNewPosition);
	
	// RELATIVO A LA COMUNICACION CON AIBO ////////////////////////
	// handles all the network communication with Aibo
    NetworkConnector *aiboConnector;
    void OpenConnection();
    void CloseConnection();
    // displays error message (mainly networking errors)
    // ErrorIDs are declared in NetworkConnector.h
    void ShowError(ErrorID error);
    // the same as above plus connection closure
    void ShowErrorAndDisconnect(ErrorID error);

};

#endif

