#ifndef NETWORKCONNECTOR_H
#define NETWORKCONNECTOR_H


typedef int ErrorID;

enum
{
    SUCCESS = 0,
    ALREADY_CONNECTED,
    NOT_CONNECTED,
    GENERAL,
    HOST,
    REFUSED,
    UNKNOWN_COMMAND,
    SENDING_COMMAND,
    RECEIVING_ANSWER,
    OPENING_FILE,
    READING_FILE,
    WRITING_FILE,
    NOT_ON_AIBO,
    NOT_MTN,
    FILE_NAME,
    FILE_SIZE,
    NUMBER_TRUNCATED_65535
};

// error messages correspondig to ErrorIDs
static const char* const ERROR_MESSAGE[] = {
    "Everything went fine",                      // SUCCESS
    "Already connected to Aibo",                 // ALREADY_CONNECTED
    "Not connected to Aibo",                     // NOT_CONNECTED
    "Internal problem",                          // GENERAL  
    "No route to Aibo",                          // HOST
    "Aibo refused connection",                   // REFUSED
    "Unknown command",                           // UNKNOWN_COMMAND
    "Lost connection to Aibo",                   // SENDING_COMMAND
    "Lost connection to Aibo",                   // RECEIVING_ANSWER
    "Could not open file",                       // OPENING_FILE
    "Could not read from file",                  // READING_FILE
    "Could not write to file",                   // WRITING_FILE
    "File not yet transferred to Aibo",          // NOT_ON_AIBO
    "File must have .MTN extension",             // NOT_MTN
    "Filename can be at most 8 letters and must have .MTN extension", // FILE_NAME
    "File is too big for transmission",          // FILE_SIZE
    "The entered number was truncated to 65535"  // NUMBER_TRUNCATED_65535
};


class NetworkConnector
{
public:

    // Constructor
    NetworkConnector(void);

    // Establishes connection with Aibo
    ErrorID Connect(const char *address);
    
    // Terminates connection with Aibo
    ErrorID Disconnect();
    
    bool IsConnected();
    
    bool IsDisconnected();
    
    // used to send commands of type BasicSetCommand to Aibo
    ErrorID Command(const char command, const unsigned char identifier, 
                    const short value);

    // used to send commands of type FileCommand to Aibo
    // path is only used internally and not sent to Aibo
    ErrorID DataCommand(const char command, const char* filename, 
                        const char* path, const unsigned short option);
    
    // used to send commands of type BasicGetCommand to Aibo    
    // value will contain returned data
    ErrorID GetCommand(const char command, const unsigned char identifier, 
                       short& value);
    
    // used to send commands of type BasicGetCommand to Aibo   
    // to which Aibo answers with MultipleValueAnswer 
    // value will contain returned data as an array
    // nbrValue is only used internally to specify the number of expected
    // entries in the array
    ErrorID GetAllCommand(const char command, const unsigned char identifier, 
                          short* value, int nbrValue);

    
private:

    bool connected;
    
    // contains the file descriptor referencing the communication socket
    int commandSender;

};

#endif // NETWORKCONNECTOR_H


